package fr.lri.insitu.FlowStates.examples.graphicalEditor;

import java.awt.geom.Point2D;

import javax.swing.JFrame;

import fr.lri.insitu.FlowStates.IConEnvironnement;
import fr.lri.insitu.FlowStates.examples.graphicalEditor.events.PanOn;
import fr.lri.insitu.FlowStates.examples.menus.GenericTrackingMenu;
import fr.lri.insitu.FlowStates.sm.IConStateMachine;
import fr.lri.insitu.FlowStates.transitions.Switch;
import fr.lri.swingstates.sm.State;
import fr.lri.swingstates.sm.Transition;

public class MainGraphicalEditor {

	public static void main(String[] args) {
		ZoomableCanvas canvas = new ZoomableCanvas(600, 400);
		
		canvas.newRectangle(new Point2D.Double(20, 20), 30, 40);
		canvas.newRectangle(new Point2D.Double(200, 200), 60, 40);
		canvas.newRectangle(new Point2D.Double(20, 200), 30, 40);
		canvas.newRectangle(new Point2D.Double(200, 20), 30, 40);
		new Palette(canvas);
		
		JFrame frame = new JFrame();
		IConStateMachine smZoom = new ZoomInteraction("zoom", canvas);
		final IConStateMachine smPan = new PanInteraction("pan", canvas);
		final Palette palette = new Palette(canvas);
		
		IConEnvironnement.addStateMachine(smZoom, frame, canvas);
		IConEnvironnement.addStateMachine(smPan, frame, canvas);
		IConEnvironnement.addStateMachine(palette.getInteractionPalette(), frame, canvas);
		IConEnvironnement.addStateMachine(palette.getInteractionToolglass(), frame, canvas);
//		smPan.suspend();
		
		// Because both state machines use the same event type (RubberBand)
		IConStateMachine mode = new IConStateMachine("modePanning", canvas) {
			State panOff = new State() {
				Transition startPan = new Switch(PanOn.class, SWITCH_ON, ">> panOn") {
					public void action() {
						smPan.resume();
						palette.getInteractionPalette().suspend();
					}
				};
			};
		
			State panOn = new State() {
				Transition stopPan = new Switch(PanOn.class, SWITCH_OFF, ">> panOff") {
					public void action() {
						smPan.suspend();
						palette.getInteractionPalette().resume();
					}
				};
			};
		};
		IConEnvironnement.addStateMachine(mode, frame, canvas);
		new GenericTrackingMenu(canvas, frame, 60, 20, new String[] {"pan", "zoom", "pan", "zoom"});
		canvas.setAntialiased(true);
		
		frame.getContentPane().add(canvas);
		frame.pack();
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		canvas.requestFocus();
		
		IConEnvironnement.start("IC files/graphical-editor.ic");
	}
	
}
