package fr.lri.insitu.FlowStates.examples.graphicalEditor;

import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;

import fr.lri.swingstates.canvas.CEllipse;
import fr.lri.swingstates.canvas.CPolyLine;
import fr.lri.swingstates.canvas.CRectangle;
import fr.lri.swingstates.canvas.CShape;
import fr.lri.swingstates.canvas.Canvas;

public class ZoomableCanvas extends Canvas {

	protected CShape sceneParent;
	
	public ZoomableCanvas(int w, int h) {
		super(w, h);
		sceneParent = (CShape)newRectangle(0, 0, 1, 1).setDrawable(false).setPickable(false);
	}
	
	public CRectangle newRectangle(Point2D pt) {
		CRectangle rect = newRectangle(pt, 1, 1);
		rect.setParent(sceneParent);
		return rect;
	}
	
	public CRectangle newRectangle(Point2D pt, double w, double h) {
		CRectangle rect = super.newRectangle(pt,w, h);
		rect.setParent(sceneParent);
		return rect;
	}
	
	public CEllipse newEllipse(Point2D pt) {
		CEllipse ell = newEllipse(pt, 1, 1);
		ell.setParent(sceneParent);
		return ell;
	}
	
	public CPolyLine newFreeHand(Point2D pt) {
		CPolyLine line = (CPolyLine)newPolyLine(pt).setFilled(false);
		line.setParent(sceneParent);
		return line;
	}
	
	public ZoomableCanvas resetZoom() {
		sceneParent.translateTo(0, 0);
		sceneParent.scaleBy(1/getZoomFactor());
		return this;
//		return zoomBy(new Point2D.Double(getWidth()/2, getHeight()/2), 1/getZoomFactor());
	}
	
	public ZoomableCanvas zoomBy(Point2D zoomCenter, double zoomFactor) {
		Point2D unchanged = sceneParent.getAbsTransform().transform(zoomCenter, new Point2D.Double());
		sceneParent.scaleBy(zoomFactor, zoomFactor);
		Point2D changed = sceneParent.getAbsTransform().transform(zoomCenter, new Point2D.Double());
		sceneParent.translateBy(unchanged.getX() - changed.getX(), unchanged.getY() - changed.getY());
		return this;
	}
	
	public double getZoomFactor() {
		return sceneParent.getScaleX();
	}
	
	public ZoomableCanvas panBy(double deltaScreenX, double deltaScreenY) {
		double dx = deltaScreenX / sceneParent.getScaleX();
		double dy = deltaScreenY / sceneParent.getScaleY();
		sceneParent.translateBy(dx, dy);
		return this;
	}
	
	public Point2D getCoordinates(Point2D pointInCanvas) {
		Point2D transformed = new Point2D.Double(0, 0);
		try {
			return sceneParent.getAbsTransform().inverseTransform(pointInCanvas, transformed);
		} catch (NoninvertibleTransformException e) {
			e.printStackTrace();
			return pointInCanvas;
		}
	}
	
}
