package fr.lri.insitu.FlowStates.examples.menus;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Paint;
import java.awt.PaintContext;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;

import fr.lri.swingstates.canvas.CEllipse;
import fr.lri.swingstates.canvas.CPolyLine;
import fr.lri.swingstates.canvas.CShape;
import fr.lri.swingstates.canvas.CText;
import fr.lri.swingstates.canvas.Canvas;

public class CircularMenuLayout extends MenuLayout {
	protected int radius = 100;
	protected int innerRadius = 30;
	protected int shadowOffset = 5;
	
	protected boolean gradient = true;
	
	protected String[] items;
	
	class RoundGradientPaint implements Paint {
	    protected Point2D point;

	    protected Point2D mRadius;

	    protected Color mPointColor, mBackgroundColor;

	    public RoundGradientPaint(double x, double y, Color pointColor,
	        Point2D radius, Color backgroundColor) {
	      if (radius.distance(0, 0) <= 0)
	        throw new IllegalArgumentException("Radius must be greater than 0.");
	      point = new Point2D.Double(x, y);
	      mPointColor = pointColor;
	      mRadius = radius;
	      mBackgroundColor = backgroundColor;
	    }

	    public PaintContext createContext(ColorModel cm, Rectangle deviceBounds,
	        Rectangle2D userBounds, AffineTransform xform, RenderingHints hints) {
	      Point2D transformedPoint = xform.transform(point, null);
	      Point2D transformedRadius = xform.deltaTransform(mRadius, null);
	      return new RoundGradientContext(transformedPoint, mPointColor,
	          transformedRadius, mBackgroundColor);
	    }

	    public int getTransparency() {
	      int a1 = mPointColor.getAlpha();
	      int a2 = mBackgroundColor.getAlpha();
	      return (((a1 & a2) == 0xff) ? OPAQUE : TRANSLUCENT);
	    }
	  }
	  public class RoundGradientContext implements PaintContext {
	    protected Point2D mPoint;

	    protected Point2D mRadius;

	    protected Color color1, color2;

	    public RoundGradientContext(Point2D p, Color c1, Point2D r, Color c2) {
	      mPoint = p;
	      color1 = c1;
	      mRadius = r;
	      color2 = c2;
	    }

	    public void dispose() {
	    }

	    public ColorModel getColorModel() {
	      return ColorModel.getRGBdefault();
	    }

	    public Raster getRaster(int x, int y, int w, int h) {
	      WritableRaster raster = getColorModel().createCompatibleWritableRaster(
	          w, h);

	      int[] data = new int[w * h * 4];
	      for (int j = 0; j < h; j++) {
	        for (int i = 0; i < w; i++) {
	          double distance = mPoint.distance(x + i, y + j);
	          double radius = mRadius.distance(0, 0);
	          double ratio = distance / radius;
	          if (ratio > 1.0)
	            ratio = 1.0;

	          int base = (j * w + i) * 4;
	          data[base + 0] = (int) (color1.getRed() + ratio
	              * (color2.getRed() - color1.getRed()));
	          data[base + 1] = (int) (color1.getGreen() + ratio
	              * (color2.getGreen() - color1.getGreen()));
	          data[base + 2] = (int) (color1.getBlue() + ratio
	              * (color2.getBlue() - color1.getBlue()));
	          data[base + 3] = (int) (color1.getAlpha() + ratio
	              * (color2.getAlpha() - color1.getAlpha()));
	        }
	      }
	      raster.setPixels(0, 0, w, h, data);

	      return raster;
	    }
	  }
	
	public CircularMenuLayout(Canvas canvas, int radius, int innerRadius, boolean gradient, String[] items) {
		super(canvas);
/*		parent = canvas.newEllipse(-5, -5, 10, 10);
		parent.addTag(parentShape);
		
		CText labelItem;
		CEllipse bgItem;
		
		for(int i = items.length; i > 0; i--) {
			bgItem = canvas.newEllipse(-i * radiusRing, -i * radiusRing, 2 * i*radiusRing, 2 * i*radiusRing);
			bgItem.addTag(tagWhole).addTag(tagBg).addTag(new MenuItem(items[i-1])).setFillPaint(MenuLayout.BG_COLOR).setOutlinePaint(MenuLayout.BORDER_COLOR);
			labelItem = (CText) canvas.newText(0, 0, items[i-1], MenuLayout.FONT)
			.setPickable(false).addTag(tagWhole).addTag(tagLabels)
			.setReferencePoint(0.5, 0.5).translateTo(bgItem.getCenterX(), bgItem.getMinY() + 5);
			parent.addChild(labelItem).addChild(bgItem);
		}

		parent.addTag(tagWhole);
		tagLabels.aboveAll();*/
		this.radius = radius;
		this.innerRadius = innerRadius;
		this.gradient = gradient;
		this.gradient = gradient;
		this.items = items;
		computeShapes();
	}
	
	private void computeShapes() {
		canvas.removeShapes(tagWhole);
	    Ellipse2D e = new Ellipse2D.Double(-1*radius, -1*radius, radius*2, radius*2);
	    Ellipse2D ie = new Ellipse2D.Double(-1*innerRadius, -1*innerRadius, innerRadius*2, innerRadius*2);
	    Area shape = new Area(e);
	    shape.subtract(new Area(ie));
	    AffineTransform ts = AffineTransform.getTranslateInstance(4,5);
	    Area shadow = new Area(ts.createTransformedShape(shape));
	    shadow.subtract(shape);
	    int nitems = items.length;
	    Shape zones[] = new Shape[nitems];
	    if (nitems == 1) {
	      GeneralPath z = new GeneralPath(GeneralPath.WIND_EVEN_ODD);
	      z.append(e, false);
	      z.append(ie, false);
	      zones[0] = z;
	    } else {
	      double dgap = ((double) 360 / (double) nitems);
	      double dgaprad = (double) 2*Math.PI / (double) nitems;
	      Arc2D a, ai;
	      GeneralPath z;
	      AffineTransform t = AffineTransform.getRotateInstance(-dgaprad, 0, 0);
	      Point2D start = new Point2D.Double(innerRadius, 0);
	      Point2D end = new Point2D.Double(radius, 0);
	      for (int i = 0; i < nitems; i++) {
	        z = new GeneralPath(GeneralPath.WIND_EVEN_ODD);
	        z.reset();
	        z.moveTo((float)start.getX(), (float)start.getY());
	        a = new Arc2D.Double(e.getBounds2D(), i*dgap, dgap, Arc2D.OPEN);
	        ai = new Arc2D.Double(ie.getBounds2D(), i*dgap, dgap, Arc2D.OPEN);
	        z.lineTo((float)a.getStartPoint().getX(), (float)a.getStartPoint().getY());
	        z.append(a, true);
	        z.lineTo((float)ai.getEndPoint().getX(), (float)ai.getEndPoint().getY());
	        z.append(ai, false);
	        z.moveTo((float)ai.getStartPoint().getX(), (float)ai.getStartPoint().getY());
	        z.lineTo((float)start.getX(), (float)start.getY());
	        z.closePath();
	        zones[i] = z;
	        start = t.transform(start, null);
	        end = t.transform(end, null);
	      }
	    }

	    parent = canvas.newEllipse(-5,-5, 10, 10);
		parent.addTag(parentShape).addTag(tagBg).setFillPaint(backgroundColor);
		parent.addTo(canvas);
		
		Paint bgPaint;
		if (gradient)
			bgPaint = new RoundGradientPaint(0, 0, Color.WHITE, new Point2D.Double(innerRadius, radius), backgroundColor);
		else
			bgPaint = backgroundColor;
		BasicStroke sectStroke = new BasicStroke(4f);
	
		CShape shad = new CShape(shadow);
		shad.addTo(canvas);
		shad.setOutlined(false).setFillPaint(shadowColor)
		//.addTag(tagBg)
		.addTag(tagWhole);
		parent.addChild(shad);
	    
	    bgShape = new CShape(shape);
		bgShape.addTo(canvas);
		bgShape.addTag(tagWhole)
		//.addTag(tagBg)
		.setOutlined(false);
		bgShape.setFillPaint(bgPaint);
		parent.addChild(bgShape);
		
		for (int i = 0; i < nitems; i++) {
		    double dgaprad = (double) 2*Math.PI / (double) nitems;
		    AffineTransform t = AffineTransform.getRotateInstance(dgaprad/2d, 0, 0);
			CShape bgItem = new CShape(t.createTransformedShape(zones[i]));
			bgItem.addTo(canvas)
			.addTag(tagWhole).addTag(tagItemBg).addTag(new MenuItem(items[i]))
			.setFillPaint(itemsBackgroundColor).setOutlinePaint(borderColor).setStroke(sectStroke);
			CText labelItem = (CText) canvas.newText(0, 0, items[i], MenuLayout.FONT)
			.setPickable(false).addTag(tagWhole).addTag(tagLabels)
			.setReferencePoint(0.5, 0.5).translateTo(bgItem.getCenterX(), bgItem.getCenterY())
			.setFillPaint(fontColor);
			parent.addChild(labelItem).addChild(bgItem);
		}
		
		parent.addTag(tagWhole);
		tagLabels.aboveAll();
	}
	
	public void setBackgroundColor(Color backgroundColor) {
		this.backgroundColor = backgroundColor;
		Paint bgPaint;
		if (gradient)
			bgPaint = new RoundGradientPaint(0, 0, Color.WHITE, new Point2D.Double(innerRadius, radius), backgroundColor);
		else
			bgPaint = backgroundColor;
		bgShape.setFillPaint(bgPaint);
	}
	
	public int getDeadZoneRadius() {
		return innerRadius;
	}
	
	public int getRadius() {
		return radius;
	}
	
	public void setRadius(int r) {
		radius = r;
		computeShapes();
	}
}
