package fr.lri.insitu.FlowStates.examples.menus;

import java.awt.geom.Point2D;
import java.util.EventObject;

import fr.lri.insitu.FlowStates.events.OutSlotEvent;
import fr.lri.insitu.FlowStates.examples.menus.events.CommandMove;
import fr.lri.insitu.FlowStates.examples.menus.events.Control;
import fr.lri.insitu.FlowStates.examples.menus.events.InRange;
import fr.lri.insitu.FlowStates.sm.IConStateMachine;
import fr.lri.insitu.FlowStates.transitions.Switch;
import fr.lri.insitu.FlowStates.transitions.SwitchOnPosition;
import fr.lri.insitu.FlowStates.transitions.SwitchOnTag;
import fr.lri.swingstates.canvas.Canvas;
import fr.lri.swingstates.canvas.transitions.EventOnTag;
import fr.lri.swingstates.sm.State;
import fr.lri.swingstates.sm.Transition;
import fr.lri.swingstates.sm.transitions.EventOnPosition;

public class GenericTrackingMenuSM extends IConStateMachine {
	
	CircularMenuLayout menu;
	
	MenuItem currentItem;
	Point2D lastPosition, startPosition;
	
	String[] properties = {"Radius"};
	
	public GenericTrackingMenuSM(Canvas canvas, CircularMenuLayout menu) {
		super("trackingMenu", canvas);
		this.menu = menu;
	}
	
	@Override
	public String[] getProperties() {
		return properties;
	}
	
	public int getRadius() {
		return menu.getRadius();
	}
	
	public void setRadius(int r) {
		menu.setRadius(r);
	}
	
	public Class<? extends OutSlotEvent>[] getOutputTypes() {
		return new Class[] {CommandMove.class};
	}
	
	State outOfRange = new State() {
		Transition startTrackingInMenu = new SwitchOnTag(MenuItem.class, InRange.class, SWITCH_ON, ">> tracking") { };
		Transition startTrackingOutMenu = new SwitchOnPosition(InRange.class, SWITCH_ON, ">> tracking") {
			public void action() {
				menu.showMenu(getPoint());
			}
		};
	};
	State tracking = new State() {
		Transition stopTracking = new Switch(InRange.class, SWITCH_OFF, ">> outOfRange") {
			public void action() {
				menu.removeHighlight();
				menu.hideMenu();
			}
		};
		
		Transition startControl = new SwitchOnTag(MenuItem.class, Control.class, SWITCH_ON, ">> touching") {
			public void action() {
				currentItem = (MenuItem)getTag();
				lastPosition = getPoint();
				startPosition = lastPosition;
			}
		};
		
		// tracking menu does not move, highlight menu items
		Transition moveInMenu = new EventOnTag(MenuItem.class, Control.class) {
			public void action() {
				if(currentItem == getTag()) return;
				menu.removeHighlight();
				currentItem = (MenuItem)getTag();
				currentItem.setFillPaint(menu.getHiliteColor());
			}
		};
		Transition moveOutMenu = new EventOnPosition(Control.class) {
			double distanceFromMenuCenter;
			public boolean guard () {
				distanceFromMenuCenter = menu.getCurrentPosition().distance(getPoint());
				return distanceFromMenuCenter > menu.getRadius();
			}
			public void action() {
				double sinAngleToCenter = (getPoint().getY() - menu.getCurrentPosition().getY()) / distanceFromMenuCenter;
				double angleToCenter = Math.asin(sinAngleToCenter);
				double distanceToOutline = distanceFromMenuCenter - menu.getRadius();
				double dx = distanceToOutline * Math.cos(angleToCenter);
				if((getPoint().getX() - menu.getCurrentPosition().getX()) < 0) dx = -dx;
				double dy = distanceToOutline * sinAngleToCenter;
				
				menu.moveBy(dx, dy);
			}
		};
	};
	State touching = new State() {
		Transition stopControl = new SwitchOnPosition(Control.class, SWITCH_OFF, ">> tracking") {
			public void action() {
				menu.showMenu(getPoint());
				fireEvent(new CommandMove("", getPoint(), 0, 0, 0, 0));
			}
		};
		
		// tracking menu does not move
		Transition control = new EventOnPosition(Control.class) {
			public void action() {
				if (currentItem != null) {
					fireEvent(new CommandMove(currentItem.getName(), getPoint(), getPoint().getX() - lastPosition.getX(), getPoint().getY() - lastPosition.getY(), startPosition.getX(), startPosition.getY()));
					lastPosition = getPoint();
				}
			}
		};
	};

}
