package fr.lri.insitu.FlowStates.examples.menus;

import javax.swing.JFrame;

import fr.lri.insitu.FlowStates.IConEnvironnement;
import fr.lri.insitu.FlowStates.events.OutSlotEvent;
import fr.lri.insitu.FlowStates.examples.menus.events.Command;
import fr.lri.insitu.FlowStates.examples.menus.events.MMenu;
import fr.lri.insitu.FlowStates.sm.IConStateMachine;
import fr.lri.insitu.FlowStates.transitions.SwitchOnPosition;
import fr.lri.swingstates.canvas.Canvas;
import fr.lri.swingstates.canvas.transitions.EventOnTag;
import fr.lri.swingstates.sm.State;
import fr.lri.swingstates.sm.Transition;
import fr.lri.swingstates.sm.transitions.TimeOut;

public class MarkingMenu {
	protected Canvas canvas;
	protected MenuLayout menu;
	
	public MarkingMenu(Canvas canvas, JFrame frame) {
		this.canvas = canvas;
		
		String[] items = {"new", "cut", "copy", "paste"};
		menu = new CircularMenuLayout(canvas, 100, 30, true, items);
		
		IConStateMachine sm = new IConStateMachine("menu", canvas) {
			MenuItem currentItem;
			
			public Class<? extends OutSlotEvent>[] getOutputTypes() {
				return new Class[] {Command.class};
			}
			
			State menuOff = new State() {
				public void enter() {
					menu.hideMenu();
					currentItem = null;
				}
				Transition menuOn = new SwitchOnPosition(MMenu.class, SWITCH_ON, ">> menuOn") {
					public void action() {
						menu.removeHighlight();
						menu.activeMenu(getPoint());
						armTimer(300, false);
					}
				};
			};
			State menuOn = new State() {
				Transition showMenu = new TimeOut() {
					public void action() {
						menu.showMenu();
					}
				};
				Transition select = new SwitchOnPosition(MMenu.class, SWITCH_OFF, ">> menuOff") {
					public void action() {
						if (currentItem != null)
							fireEvent(new Command(currentItem.getName()));
						System.out.println("command "+(currentItem != null ? currentItem.getName() : "_"));
					}
				};
				Transition enterOnItem = new EventOnTag(MenuItem.class, MMenu.class) {
					public void action() {
						disarmTimer();
						armTimer(300, false);
						if(currentItem == getTag()) return;
						menu.removeHighlight();
						currentItem = (MenuItem)getTag();
						currentItem.setFillPaint(menu.getHiliteColor());
					}
				};
				Transition enterOnCenter = new EventOnTag(menu.getParentShape(), MMenu.class) {
					public void action() {
						disarmTimer();
						armTimer(300, false);
						currentItem = null;
					}
				};
			};
		};
		IConEnvironnement.addStateMachine(sm, frame, canvas);
		
		
	}
	
	public static void main(String[] args) {
		Canvas canvas = new Canvas (600, 400);
		canvas.setAntialiased(true);
		JFrame frame = new JFrame();
		new MarkingMenu(canvas, frame);
		frame.getContentPane().add(canvas);
		frame.pack();
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		
		IConEnvironnement.start("marking-menu.ic");
		
	}
}
