package fr.lri.insitu.FlowStates.examples.menus;

import java.util.EventObject;

import javax.swing.JFrame;

import fr.lri.insitu.FlowStates.IConEnvironnement;
import fr.lri.insitu.FlowStates.events.OutSlotEvent;
import fr.lri.insitu.FlowStates.examples.menus.events.Command;
import fr.lri.insitu.FlowStates.examples.menus.events.MenuOn;
import fr.lri.insitu.FlowStates.examples.menus.events.SelectLocation;
import fr.lri.insitu.FlowStates.sm.IConStateMachine;
import fr.lri.insitu.FlowStates.transitions.SwitchOnPosition;
import fr.lri.insitu.FlowStates.transitions.SwitchOnTag;
import fr.lri.swingstates.canvas.Canvas;
import fr.lri.swingstates.canvas.transitions.EventOnTag;
import fr.lri.swingstates.sm.State;
import fr.lri.swingstates.sm.Transition;
import fr.lri.swingstates.sm.transitions.Event;

public class TearOffMenu {

	protected Canvas canvas;
	protected MenuLayout menu;
	
	public TearOffMenu(Canvas canvas, JFrame frame) {
		this.canvas = canvas;
		
		String[] items = {"new", "cut", "copy", "paste"};
		menu = new LinearMenuLayout(canvas, items);
		
		final IConStateMachine sm = new IConStateMachine("menu", canvas) {
			MenuItem currentItem;
			
			public Class<? extends OutSlotEvent>[] getOutputTypes() {
				return new Class[] {Command.class};
			}
			
			State menuOff = new State() {
				public void enter() {
					menu.hideMenu();
					currentItem = null;
				}
				Transition menuOn = new SwitchOnPosition(MenuOn.class, true, ">> menuOn") {
					public void action() {
						menu.removeHighlight();
						menu.showMenu(getPoint());
					}
				};
			};
			State menuOn = new State() {
				Transition enterOnItem = new EventOnTag(MenuItem.class, SelectLocation.class, ">> onItem") {
					public void action() {
						currentItem = (MenuItem)getTag();
						currentItem.setFillPaint(menu.getHiliteColor());
					}
				};
				Transition cancel = new SwitchOnPosition(MenuOn.class, false, ">> menuOff") { };
			};
			State onItem = new State() {
				Transition select = new SwitchOnTag(MenuItem.class, MenuOn.class, false, ">> menuOff") {
					public void action() {
						fireEvent(new Command(currentItem.getName()));
						System.out.println("select command "+currentItem.getName());
					}
				};
				Transition hilite = new EventOnTag(MenuItem.class, SelectLocation.class) {
					public void action() {
						if(currentItem == getTag()) return;
						menu.removeHighlight();
						currentItem = (MenuItem)getTag();
						currentItem.setFillPaint(menu.getHiliteColor());
					}
				};
				Transition outMenu = new Event(SelectLocation.class, ">> menuOn") { };
				public void leave() {
					menu.removeHighlight();
				}
			};
		};
		IConEnvironnement.addStateMachine(sm, frame, canvas);
		
		
	}
	
	public static void main(String[] args) {
		Canvas canvas = new Canvas (600, 400);
		JFrame frame = new JFrame();
		new TearOffMenu(canvas, frame);
		frame.getContentPane().add(canvas);
		frame.pack();
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		
		IConEnvironnement.start("tearoff-2.ic");
		
	}
}
