package fr.lri.insitu.FlowStates.examples.toolglass;

import java.awt.geom.Point2D;

import javax.swing.JFrame;

import fr.lri.insitu.FlowStates.IConEnvironnement;
import fr.lri.insitu.FlowStates.examples.toolglass.events.TensionOn;
import fr.lri.insitu.FlowStates.examples.toolglass.events.ToolsLocation;
import fr.lri.insitu.FlowStates.sm.IConStateMachine;
import fr.lri.insitu.FlowStates.transitions.Switch;
import fr.lri.insitu.FlowStates.transitions.SwitchOnTag;
import fr.lri.swingstates.canvas.CExtensionalTag;
import fr.lri.swingstates.canvas.CRectangularShape;
import fr.lri.swingstates.canvas.Canvas;
import fr.lri.swingstates.sm.State;
import fr.lri.swingstates.sm.Transition;
import fr.lri.swingstates.sm.transitions.EventOnPosition;

abstract class Tool extends CExtensionalTag {

	public Tool() {
		super();
	}
	
	public abstract CRectangularShape getShape(double x, double y);
	
}


public class Toolglass {

	protected IConStateMachine shapeCreation, palettePosition;
	protected Canvas canvas;
	protected Palette palette;
	
	public Toolglass(Canvas canvas, JFrame frame) {
		this.canvas = canvas;
		
		palette = new Palette(canvas);
		palette.setTransparencyLevel(0.3f);
		
		palettePosition = new IConStateMachine("toolsPosition", canvas) {
			State idle = new State() {
				Transition moveTools = new EventOnPosition(ToolsLocation.class) {
					public void action() {
						palette.moveTo(getPoint().getX(), getPoint().getY(), 0.5, 0.5);
					}
				};
			};
		};
		
		shapeCreation = new IConStateMachine("shapeCreation", canvas) {
			CRectangularShape currentShape = null;
			Point2D p1, p2;
			
			State noTool = new State() {
				Transition on = new SwitchOnTag(Tool.class, TensionOn.class, SWITCH_ON, ">> draw") {
					public void action() {
						currentShape = ((Tool)getTag()).getShape(getPoint().getX(), getPoint().getY());
						p1 = getPoint();
						p2 = getPoint();
						System.out.println("create "+getTag().getClass().getSuperclass().getSimpleName());
					}
				};
			};
			
			State draw = new State() {
				Transition off = new Switch(TensionOn.class, SWITCH_OFF, ">> noTool") { };
				
				Transition rubberBand = new EventOnPosition(TensionOn.class) {
					public void action() {
						p2 = getPoint();
						currentShape.setDiagonal(p1, p2);
					}
				};
			};
		};
		
		IConEnvironnement.addStateMachine(palettePosition, frame, canvas);
		IConEnvironnement.addStateMachine(shapeCreation, frame, canvas);
		
	}
	
	public static void main(String[] args) {
		Canvas canvas = new Canvas (600, 400);
		JFrame frame = new JFrame();
		Toolglass toolglass = new Toolglass(canvas, frame);
		frame.getContentPane().add(canvas);
		frame.pack();
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		
		IConEnvironnement.start("toolglass.ic");
		

	}

}
