package fr.lri.insitu.FlowStates.sm;

//import java.lang.reflect.Method;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import fr.emn.reactiveinput.devices.UniqueNames;
import fr.lri.insitu.FlowStates.device.DSMDevice;
import fr.lri.insitu.FlowStates.device.InSlotsGroup;
import fr.lri.insitu.FlowStates.events.IConPositionEvent;
import fr.lri.insitu.FlowStates.events.InSlotEvent;
import fr.lri.insitu.FlowStates.events.OutSlotEvent;
import fr.lri.swingstates.canvas.CStateMachine;
import fr.lri.swingstates.canvas.Canvas;
import fr.lri.swingstates.canvas.transitions.EventOnShape;
import fr.lri.swingstates.events.VirtualEvent;
import fr.lri.swingstates.sm.State;
import fr.lri.swingstates.sm.Transition;
import fr.lri.swingstates.sm.transitions.Event;

public class IConStateMachine extends CStateMachine {

	private UniqueNames uNames = new UniqueNames();
	
	protected String name = "";
	//private DSMDevice prototypeDevice = null;

	// Contrary to a CStateMachine, an IConStateMachine can only be attached to a canvas
	protected Canvas canvas;

	public static boolean SWITCH_ON  = true;
	public static boolean SWITCH_OFF = false;
	
	public IConStateMachine() {
		super();
	}
	
	public String[] getProperties() {
		return null;
	}
	
	public IConStateMachine(String name, Canvas c) {
		super(c);
		this.name = uNames.add(name);
		canvas = c;
		//prototypeDevice = new DSMDevice(this);
	}

	public String getName() {
		return name;
	}

	public void setName(String n) {
		name = uNames.add(n);
	}
	
/*	public DSMDevice getPrototypeDevice() {
		return prototypeDevice;
	}*/

	public Canvas getCanvas() {
		return canvas;
	}
	
	public void setCanvas(Canvas c) {
		canvas = c;
	}

	public Class<? extends OutSlotEvent>[] getOutputTypes() {
		return null;
	}

	public Event[] getIConTransitions() {
		Vector<Event> transitions = new Vector<Event>();
		for (State st : this.getAllStates()) {
			for (Transition tr : st.getTransitions()) {
				if (tr instanceof Event && InSlotEvent.class.isAssignableFrom(((Event)tr).getClassEvent())) {
					transitions.add((Event)tr);
					/*System.out.println("IconTransition");
					Class<?> evClass = ((Event)tr).getClassEvent();
					Method meth[] = evClass.getMethods();
					for (Method m : meth) {
						if (m.getName().startsWith("getSlot")) {
							System.out.println("    slot:" + m.getName());
						}
					}*/
				}
			}
		}
		Event ar[] = new Event[] {};
		return transitions.toArray(ar);
	}

	public Transition processIconEvents(DSMDevice dsm) throws IllegalArgumentException, SecurityException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
		if(!inited) initStatesAndTransitions();
		if(!isActive()) return null;
		LinkedList<Transition> trans = currentState.getTransitions();
		Transition hasFired = null;
		if(trans!=null){
			// To avoid building two instances of the same event type
			// because it causes troubles for programming events to 
			// handle value changes using static variables
			LinkedList<VirtualEvent> firedEvents = new LinkedList<VirtualEvent>();

			for(Iterator<Transition> i = trans.iterator(); i.hasNext(); ){
				
				if (hasFired != null) {
					break;
				}
				Transition t = i.next();
				if (!(t instanceof Event) || ((Event)t).getClassEvent() == null) continue;
				InSlotsGroup slotsGroup = dsm.getInSlotsGroup(t);
				if(slotsGroup == null) continue;
				VirtualEvent toFire = null;

				boolean alreadyFired = false;
				for (Iterator<VirtualEvent> iterator = firedEvents.iterator(); iterator.hasNext();) {
					VirtualEvent next = iterator.next();
					if(slotsGroup.getEvClass().equals(next.getClass())) {
						toFire = next;
						alreadyFired = true;
						break;
					}
				}
				
				if(toFire == null) {
					toFire = slotsGroup.getEvent();
					if(toFire != null) {
						firedEvents.add(toFire);
						toFire.setSource(getCanvas());
					}
				}

				IConPositionEvent posEvent = null;
				if (toFire instanceof IConPositionEvent) {
					posEvent = ((IConPositionEvent)toFire);
					if(!alreadyFired) posEvent.convertFromScreen(canvas);
					if(!canvas.hasFocus()) continue;
				}

				if((EventOnShape.class.isAssignableFrom(t.getClass())
						|| EventOnShape.class.isAssignableFrom(t.getClass()))) {
					// picking is required
					if(toFire != null && toFire instanceof IConPositionEvent
							&& ((IConPositionEvent)toFire).getShape() == null) {
						posEvent.setShape(canvas.pick(posEvent.getPoint()));
					}
				}
				
				if(toFire != null) {
					if(!((toFire instanceof InSlotEvent) && !((InSlotEvent)toFire).occurs())) {
						if(t.matches(toFire)) {
							t.setTriggeringEvent(toFire);
							if (fireTransition(t)) {
								hasFired = t;
								// When a matching transition is found, the event types involved in the new 
								// currentState are initialized. It avoids we detect changes 
								// at the next "event" that comes from bad initializations (and not
								// actually from new input)
								initCurrentState(dsm);
							}
						}
					}
					// In other cases (either this transition is not interested in this kind of event
					// or its guard blocked the event), we test the next slots (i.e. the next transition)
				} else {
					// the slots group associated to the type of event that triggers this transition
					// hasn't changed, we test the next slots (i.e. the next transition)
				}

			}
		}
		return hasFired;
	}

	/**
	 * Builds an event of each type for the current state
	 * @throws NoSuchMethodException 
	 * @throws InvocationTargetException 
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 * @throws SecurityException 
	 * @throws IllegalArgumentException 
	 */
	private void initCurrentState(DSMDevice dsm) throws IllegalArgumentException, SecurityException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
		LinkedList<Transition> trans = currentState.getTransitions();
		if(trans!=null){
			// To avoid building two instances of the same event type
			// because it causes troubles for programming events to 
			// handle value changes using static variables
			LinkedList<VirtualEvent> firedEvents = new LinkedList<VirtualEvent>();
				for(Iterator<Transition> i = trans.iterator(); i.hasNext(); ){
					Transition t = i.next();
					if (!(t instanceof Event) || ((Event)t).getClassEvent() == null) continue;
					InSlotsGroup slotsGroup = dsm.getInSlotsGroup(t);
					if(slotsGroup == null) continue;
					VirtualEvent toFire = null;

					boolean eventTypeAlreadySeen = false;
					for (Iterator<VirtualEvent> iterator = firedEvents.iterator(); iterator.hasNext();) {
						VirtualEvent next = iterator.next();
						eventTypeAlreadySeen = slotsGroup.getEvClass().equals(next.getClass());
						if(eventTypeAlreadySeen) {
							toFire = next;
							break;
						}
					}
					if(toFire == null) {
						toFire = slotsGroup.getEvent();
						if(toFire != null) firedEvents.add(toFire);
					}
				}
		}
	}
	
	public IConStateMachine copy() {
		IConStateMachine copy = this;
		if (isCopiable()) {
			try {//Doesn't work with anonymous classes... cannot be instantiated with reflexivity
				Class<? extends IConStateMachine> smClass = this.getClass();
				Class<?> enclosingClass = this.getClass().getEnclosingClass();
				if (enclosingClass != null) {
					Object outerObject = enclosingClass.newInstance();			
					Constructor<? extends IConStateMachine> constructor = smClass.getDeclaredConstructor(new Class[] {enclosingClass});
					constructor.setAccessible(true);
					copy = (IConStateMachine)constructor.newInstance(new Object[]{outerObject});
				} else {
					copy =  (IConStateMachine)this.getClass().newInstance();
				}
				copy.setCanvas(canvas);
				copy.setName(name);
			}
			catch (Exception e) {
				//e.printStackTrace();
				System.out.println("*** Warning: Can't copy StateMachine " + name);
			}
		}
		return copy;
	}

	public boolean isCopiable() {
		return true;
	}
}
